import {
  signOut,
  deleteUser,
  updateEmail,
  updateProfile,
  EmailAuthProvider,
  reauthenticateWithCredential,
} from '@firebase/auth';
import {
  doc,
  getDoc,
  setDoc,
  deleteDoc,
  collection,
} from '@firebase/firestore';
import { debounce } from 'lodash';
import { db } from '@/firebase/app';
import { auth } from '@/firebase/auth';
import classes from './MyPage.module.scss';
import { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { Input, ProfileImage } from '@/components';
import { useDocumentTitle } from '@/hooks/useDocumentTitle';
import { isValidEmail, isValidTel } from './../../utils/validation';
import { handleSignOut } from '@/utils/signOut';

export default function MyPage() {
  useDocumentTitle('슬기로운 N밥생활 | 마이 페이지');

  const navigation = useNavigate();
  const [name, setName] = useState('');
  const [email, setEmail] = useState('');
  const [address, setAddress] = useState('');
  const [isLoading, setIsLoading] = useState(true);
  const [isEditing, setIsEditing] = useState(false);
  const [phoneNumber, setPhoneNumber] = useState('');

  const user = auth.currentUser;

  useEffect(() => {
    // 사용자 정보, 전화번호 또는 주소가 존재할 경우, 로딩 상태 false로 설정
    if (user && (phoneNumber || address)) {
      setIsLoading(false);
    }
  }, [address, phoneNumber, user]);

  useEffect(() => {
    const unsub = auth.onAuthStateChanged((user) => {
      if (user) {
        const getUserRef = doc(collection(db, 'users'), user.uid);
        getDoc(getUserRef).then((doc) => {
          if (doc.exists()) {
            const userData = doc.data();
            setName(userData.displayName);
            setEmail(userData.email);
            setPhoneNumber(userData.phoneNumber);
            setAddress(userData.address);
            setIsLoading(false); // Firestore 문서 정보가 있으면 로딩 상태 false로 설정
          } else {
            user.providerData.forEach((profile) => {
              setName(profile.displayName);
              setEmail(profile.email);
            });
            setIsLoading(false);
          }
        });
      }
    });
    return unsub;
  }, []);

  /* ------------------------------- '회원정보수정' 클릭 ------------------------------ */
  const handleEditClick = () => {
    if (
      user.providerData[0].providerId === 'google.com' ||
      user.providerData[0].photoURL.includes('kakao')
    ) {
      alert('구글 및 카카오 사용자는 회원정보수정이 불가합니다!');
    } else {
      setIsEditing(!isEditing);
    }
  };

  /* -------------------------------- 수정 완료 클릭 -------------------------------- */
  const handleSaveClick = () => {
    /* ----- validation ----- */
    if (!name || name.trim().length < 2) {
      alert('이름은 2글자 이상 입력해야 해요');
      return;
    }

    if (!isValidEmail(email)) {
      alert('이메일 형식으로 입력해 주세요.');
      return;
    }

    if (!isValidTel(phoneNumber)) {
      alert('휴대폰 번호 01012345678 형식으로 입력해 주세요.');
      return;
    }

    /* ----- Firestore 업데이트 ----- */
    const getUserRef = doc(collection(db, 'users'), user.uid);
    setDoc(
      getUserRef,
      {
        displayName: name,
        email: email,
        phoneNumber: phoneNumber,
        address: address,
      },
      { merge: true }
    ).then(() => {
      setIsEditing(false);
    });

    /* ----- currentUser 업데이트 (displayName, email) ----- */
    updateProfile(user, { displayName: name })
      .then(() => {
        console.log('사용자 이름 변경 완료!');
      })
      .catch((error) => {
        console.log('사용자 이름 변경 실패', error);
      });

    updateEmail(user, email)
      .then(() => {
        getDoc(getUserRef).then((doc) => {
          if (doc.exists()) {
            const userProvidedPassword = doc.data().password;

            const credential = EmailAuthProvider.credential(
              user.email,
              userProvidedPassword
            );
            reauthenticateWithCredential(user, credential)
              .then(() => {
                console.log('재인증에 성공하였습니다.');
                alert('회원 정보가 수정되었습니다!');
              })
              .catch((error) => {
                console.log('재인증에 실패하였습니다.', error);
              });
          }
        });
        console.log('사용자 이메일 변경 완료!');
      })
      .catch((error) => {
        console.log('사용자 이메일 변경 실패!', error);
      });
  };

  const deleteDocument = (member: string) => {
    const user = auth.currentUser;
    deleteDoc(doc(db, 'users', user.uid));
    deleteUser(user)
      .then(() => {
        alert(`${member} 되었습니다.`);
        navigation('/');
      })
      .catch((error) => {
        console.log(`${member} 실패!`, error);
      });
  };

  /* ---------------------------------- 로그아웃 ---------------------------------- */
  const handlerSignOut = async () => {
    await handleSignOut(auth, user, navigation);
  };
  /* ---------------------------------- 회원탈퇴 ---------------------------------- */
  const handleSignDropOut = async () => {
    deleteDocument('회원 탈퇴');

    if (user.providerData[0].photoURL.includes('kakao')) {
      const { Kakao, location } = window;
      const CLIENT_ID = import.meta.env.VITE_KAKAO_API_KEY;
      const LOGOUT_REDIRECT_URI = 'http://localhost:3000';

      // Kakao API 토큰 만료 설정
      await Kakao.Auth.logout();

      // Kakao 계정 로그아웃 설정
      location.replace(
        `https://kauth.kakao.com/oauth/logout?client_id=${CLIENT_ID}&logout_redirect_uri=${LOGOUT_REDIRECT_URI}`
      );
    } else {
      navigation('/');
    }
  };

  /* -------------------------------- debounce -------------------------------- */
  const editName = debounce((e) => {
    console.log(e.target.value);
    setName(e.target.value);
  }, 500);

  const editEmail = debounce((e) => {
    console.log(e.target.value);
    setEmail(e.target.value);
  }, 500);

  const editPhoneNumber = debounce((e) => {
    console.log(e.target.value);
    setPhoneNumber(e.target.value);
  }, 500);

  const editAddress = debounce((e) => {
    console.log(e.target.value);
    setAddress(e.target.value);
  }, 500);

  if (isLoading) {
    return (
      <div role="alert" className={classes.isLoading}>
        사용자 정보 로딩 중...
      </div>
    );
  }

  return (
    <>
      <h2 className="a11yHidden">마이 페이지</h2>
      <section className={classes.myPageSection}>
        <div className={classes.myPageContainer}>
          <h3 className={classes.myPageTitle}>내 정보 관리</h3>
          <div className={classes.userContainer}>
            <ProfileImage />
            <div className={classes.inputContainer}>
              <div className={classes.userInfoContainer}>
                <div>
                  <Input
                    className={classes.inputMobile}
                    maxWidthValue={290}
                    heightValue={80}
                    labelText="Name"
                    defaultValue={user.displayName || undefined}
                    onChange={editName}
                    disabled={!isEditing}
                  />
                </div>
                <div>
                  <Input
                    className={classes.inputMobile}
                    maxWidthValue={290}
                    heightValue={80}
                    labelText="Email"
                    defaultValue={user.email}
                    onChange={editEmail}
                    disabled={!isEditing}
                  />
                </div>
              </div>
              <div className={classes.userInfoContainer}>
                <div>
                  <Input
                    className={classes.inputMobile}
                    maxWidthValue={290}
                    heightValue={80}
                    labelText="Phone"
                    defaultValue={phoneNumber}
                    onChange={editPhoneNumber}
                    disabled={!isEditing}
                  />
                </div>
                <div>
                  <Input
                    className={classes.inputMobile}
                    maxWidthValue={290}
                    heightValue={80}
                    labelText="Address"
                    defaultValue={address}
                    onChange={editAddress}
                    disabled={!isEditing}
                  />
                </div>
              </div>
            </div>
          </div>
          <div className={classes.userAbleContainer}>
            <button
              className={classes.userAbleItem}
              style={isEditing ? { color: 'red' } : { color: 'black' }}
              onClick={isEditing ? handleSaveClick : handleEditClick}
              tabIndex={0}
            >
              {isEditing ? '수정 완료' : '회원정보수정'}
            </button>
            <span>|</span>
            <button
              className={classes.userAbleItem}
              onClick={handlerSignOut}
              tabIndex={0}
            >
              로그아웃
            </button>
            <span>|</span>
            <button
              className={classes.userAbleItem}
              onClick={handleSignDropOut}
              tabIndex={0}
            >
              회원탈퇴
            </button>
          </div>
        </div>
      </section>
    </>
  );
}
